==> petsc_2.1.3.orig.tar.gz: petsc-2.1.3/src/mat/order/degree.c <==
/*$Id: degree.c,v 1.21 2001/03/23 23:22:51 balay Exp $*/
/* degree.f -- translated by f2c (version 19931217).*/

#include "petsc.h"
#include "src/mat/order/order.h"

/*****************************************************************/
/*********     DEGREE ..... DEGREE IN MASKED COMPONENT   *********/
/*****************************************************************/

/*    PURPOSE - THIS ROUTINE COMPUTES THE DEGREES OF THE NODES*/
/*       IN THE CONNECTED COMPONENT SPECIFIED BY MASK AND ROOT*/
/*       NODES FOR WHICH MASK IS ZERO ARE IGNORED.*/

/*    INPUT PARAMETER -*/
/*       ROOT - IS THE INPUT NODE THAT DEFINES THE COMPONENT.*/
/*       (XADJ, ADJNCY) - ADJACENCY STRUCTURE PAIR.*/
/*       MASK - SPECIFIES A SECTION SUBGRAPH.*/

/*    OUTPUT PARAMETERS -*/
/*       DEG - ARRAY CONTAINING THE DEGREES OF THE NODES IN*/
/*             THE COMPONENT.*/
/*       CCSIZE-SIZE OF THE COMPONENT SPECIFED BY MASK AND ROOT*/
/*    WORKING PARAMETER -*/
/*       LS - A TEMPORARY VECTOR USED TO STORE THE NODES OF THE*/
/*              COMPONENT LEVEL BY LEVEL.*/
/*****************************************************************/
#undef __FUNCT__  
#define __FUNCT__ "SPARSEPACKdegree" 
int SPARSEPACKdegree(int *root, int *xadj,int *adjncy,int *mask,int *deg,int *ccsize,int *ls)
{
    /* System generated locals */
    int i__1,i__2;

    /* Local variables */
    int ideg,node,i,j,jstop,jstrt,lbegin,lvlend,lvsize,
	    nbr;
/*       INITIALIZATION ...*/
/*       THE ARRAY XADJ IS USED AS A TEMPORARY MARKER TO*/
/*       INDICATE WHICH NODES HAVE BEEN CONSIDERED SO FAR.*/

    PetscFunctionBegin;
    /* Parameter adjustments */
    --ls;
    --deg;
    --mask;
    --adjncy;
    --xadj;

    ls[1] = *root;
    xadj[*root] = -xadj[*root];
    lvlend = 0;
    *ccsize = 1;
/*       LBEGIN IS THE POINTER TO THE BEGINNING OF THE CURRENT*/
/*       LEVEL, AND LVLEND POINTS TO THE END OF THIS LEVEL.*/
L100:
    lbegin = lvlend + 1;
    lvlend = *ccsize;
/*       FIND THE DEGREES OF NODES IN THE CURRENT LEVEL,*/
/*       AND AT THE SAME TIME, GENERATE THE NEXT LEVEL.*/
    i__1 = lvlend;
    for (i = lbegin; i <= i__1; ++i) {
	node = ls[i];
	jstrt = -xadj[node];
	jstop = (i__2 = xadj[node + 1], (int)PetscAbsInt(i__2)) - 1;
	ideg = 0;
	if (jstop < jstrt) {
	    goto L300;
	}
	i__2 = jstop;
	for (j = jstrt; j <= i__2; ++j) {
	    nbr = adjncy[j];
	    if (mask[nbr] == 0) {
		goto L200;
	    }
	    ++ideg;
	    if (xadj[nbr] < 0) {
		goto L200;
	    }
	    xadj[nbr] = -xadj[nbr];
	    ++(*ccsize);
	    ls[*ccsize] = nbr;
L200:
	    ;
	}
L300:
	deg[node] = ideg;
    }
/*       COMPUTE THE CURRENT LEVEL WIDTH. */
/*       IF IT IS NONZERO, GENERATE ANOTHER LEVEL.*/                       
    lvsize = *ccsize - lvlend;
    if (lvsize > 0) {
	goto L100;
    }
/*       RESET XADJ TO ITS CORRECT SIGN AND RETURN. */
/*       ------------------------------------------*/
    i__1 = *ccsize;
    for (i = 1; i <= i__1; ++i) {
	node = ls[i];
	xadj[node] = -xadj[node];
    }
    PetscFunctionReturn(0);
}

==> petsc_2.3.2.orig.tar.gz: petsc-2.3.2-p6/src/mat/order/degree.c <==
#define PETSCMAT_DLL

/* degree.f -- translated by f2c (version 19931217).*/

#include "petsc.h"
#include "src/mat/order/order.h"

/*****************************************************************/
/*********     DEGREE ..... DEGREE IN MASKED COMPONENT   *********/
/*****************************************************************/

/*    PURPOSE - THIS ROUTINE COMPUTES THE DEGREES OF THE NODES*/
/*       IN THE CONNECTED COMPONENT SPECIFIED BY MASK AND ROOT*/
/*       NODES FOR WHICH MASK IS ZERO ARE IGNORED.*/

/*    INPUT PARAMETER -*/
/*       ROOT - IS THE INPUT NODE THAT DEFINES THE COMPONENT.*/
/*       (XADJ, ADJNCY) - ADJACENCY STRUCTURE PAIR.*/
/*       MASK - SPECIFIES A SECTION SUBGRAPH.*/

/*    OUTPUT PARAMETERS -*/
/*       DEG - ARRAY CONTAINING THE DEGREES OF THE NODES IN*/
/*             THE COMPONENT.*/
/*       CCSIZE-SIZE OF THE COMPONENT SPECIFED BY MASK AND ROOT*/
/*    WORKING PARAMETER -*/
/*       LS - A TEMPORARY VECTOR USED TO STORE THE NODES OF THE*/
/*              COMPONENT LEVEL BY LEVEL.*/
/*****************************************************************/
#undef __FUNCT__  
#define __FUNCT__ "SPARSEPACKdegree" 
PetscErrorCode SPARSEPACKdegree(PetscInt *root, PetscInt *xadj,PetscInt *adjncy,PetscInt *mask,PetscInt *deg,PetscInt *ccsize,PetscInt *ls)
{
    /* System generated locals */
    PetscInt i__1,i__2;

    /* Local variables */
    PetscInt ideg,node,i,j,jstop,jstrt,lbegin,lvlend,lvsize,nbr;
/*       INITIALIZATION ...*/
/*       THE ARRAY XADJ IS USED AS A TEMPORARY MARKER TO*/
/*       INDICATE WHICH NODES HAVE BEEN CONSIDERED SO FAR.*/

    PetscFunctionBegin;
    /* Parameter adjustments */
    --ls;
    --deg;
    --mask;
    --adjncy;
    --xadj;

    ls[1] = *root;
    xadj[*root] = -xadj[*root];
    lvlend = 0;
    *ccsize = 1;
/*       LBEGIN IS THE POINTER TO THE BEGINNING OF THE CURRENT*/
/*       LEVEL, AND LVLEND POINTS TO THE END OF THIS LEVEL.*/
L100:
    lbegin = lvlend + 1;
    lvlend = *ccsize;
/*       FIND THE DEGREES OF NODES IN THE CURRENT LEVEL,*/
/*       AND AT THE SAME TIME, GENERATE THE NEXT LEVEL.*/
    i__1 = lvlend;
    for (i = lbegin; i <= i__1; ++i) {
	node = ls[i];
	jstrt = -xadj[node];
	jstop = (i__2 = xadj[node + 1], (PetscInt)PetscAbsInt(i__2)) - 1;
	ideg = 0;
	if (jstop < jstrt) {
	    goto L300;
	}
	i__2 = jstop;
	for (j = jstrt; j <= i__2; ++j) {
	    nbr = adjncy[j];
	    if (!mask[nbr]) {
		goto L200;
	    }
	    ++ideg;
	    if (xadj[nbr] < 0) {
		goto L200;
	    }
	    xadj[nbr] = -xadj[nbr];
	    ++(*ccsize);
	    ls[*ccsize] = nbr;
L200:
	    ;
	}
L300:
	deg[node] = ideg;
    }
/*       COMPUTE THE CURRENT LEVEL WIDTH. */
/*       IF IT IS NONZERO, GENERATE ANOTHER LEVEL.*/                       
    lvsize = *ccsize - lvlend;
    if (lvsize > 0) {
	goto L100;
    }
/*       RESET XADJ TO ITS CORRECT SIGN AND RETURN. */
/*       ------------------------------------------*/
    i__1 = *ccsize;
    for (i = 1; i <= i__1; ++i) {
	node = ls[i];
	xadj[node] = -xadj[node];
    }
    PetscFunctionReturn(0);
}
